@@ -153,7 +153,7 @@ NMexec <- function(files,file.pattern,dir,sge=TRUE,input.archive,
 
     cmd.execute <- file.psn(NMsimConf$dir.psn,"execute")
 
-    
+
     ## system.type <- getSystemType(system.type)
 
 
@@ -166,8 +166,11 @@ NMexec <- function(files,file.pattern,dir,sge=TRUE,input.archive,
 
     if(missing(nc)) nc <- NULL
     nc <- NMdata:::NMdataDecideOption("nc",nc,allow.unknown = TRUE)
+    ## not needed with NMdata 0.1.6 or 0.1.7
     if(is.null(nc)) nc <- 64
     
+    ## if(NMsimConf$method.execute=="nmsim" && nc>1){message("\nNotice: nc>1 still does not work with method.execute=\"nmsim\". Expect single-core performance. Notice there are other and most often more efficient methods to speed up simulations. See discussions on the NMsim website.")}
+
     
     ## args.psn.execute
     if(missing(args.psn.execute)) args.psn.execute <- NULL
@@ -204,8 +207,12 @@ NMexec <- function(files,file.pattern,dir,sge=TRUE,input.archive,
     if(update.only){
         ## files.exec <- findUpdated(fnExtension(files.all,"lst"))
         files.exec <- findUpdated(files.all)
+        if(length(files.exec)>0){
+            message(length(files.exec)," model(s) to be executed:\n",paste(files.exec,collapse=",\n"),"\n")
+        }
     }
-    
+
+
     for(file.mod in files.exec){
         file.mod <- NMdata:::filePathSimple(file.mod)
         if(!quiet) message(paste0("Executing ",file.mod))
@@ -236,8 +243,8 @@ NMexec <- function(files,file.pattern,dir,sge=TRUE,input.archive,
                 }
                 dir.create(dir.backup)
                 lapply(c(files.found),function(f) file.rename(
-                                                   from=file.path(rundir,f),
-                                                   to=file.path(dir.backup,f)
+                                                      from=file.path(rundir,f),
+                                                      to=file.path(dir.backup,f)
                                                   ))
                 file.copy(file.mod,dir.backup)
             } else {
@@ -254,13 +261,16 @@ NMexec <- function(files,file.pattern,dir,sge=TRUE,input.archive,
         }
 
 
-        if((sge && nc > 1)||(sge && NMsimConf$method.execute=="psn")){
+        ##if((sge && nc > 1)||(sge && NMsimConf$method.execute=="psn")){
+        if( sge ){
             if(nc>1){
-                ## file.pnm <- file.path(rundir,"NMexec.pnm")
-                ## file.pnm <- fnExtension(file.mod,"pnm")
                 file.pnm <- file.path(rundir,fnExtension(basename(file.mod),"pnm"))
                 pnm <- NMgenPNM(nc=nc,file=file.pnm)
                 files.needed <- unique(c(files.needed,pnm) )
+            } else {
+                ## for nc=1, the pnm file setup does not seem to work.
+                if(!quiet) message("sge is disabled because nc=1")
+                sge <- FALSE
             }
         }
 
@@ -284,8 +294,11 @@ NMexec <- function(files,file.pattern,dir,sge=TRUE,input.archive,
         
         if(NMsimConf$method.execute=="nmsim"){
             
-            string.cmd <- NMexecDirectory(file.mod,NMsimConf$path.nonmem,files.needed=files.needed,system.type=NMsimConf$system.type,dir.data=dir.data,clean=clean)
-            if(sge) {
+            string.cmd <- NMexecDirectory(file.mod,NMsimConf$path.nonmem,files.needed=files.needed,system.type=NMsimConf$system.type,dir.data=dir.data,clean=clean,sge,nc,pnm=pnm)
+            dir.tmp <- dirname(string.cmd)
+
+            if(F){
+                ##         if(sge) {
 
                 if(nc==1){
                     ## string.cmd <- sprintf("cd %s; qsub -terse -wd \'%s\' %s",getwd(),dirname(string.cmd),string.cmd)
@@ -300,21 +313,31 @@ NMexec <- function(files,file.pattern,dir,sge=TRUE,input.archive,
 ### executing from getwd()
                     ## string.cmd <- sprintf('cd %s; qsub -pe orte %s -V -N NMsim -j y -cwd -b y %s %s %s -background -parafile=%s [nodes]=%s' ,getwd(),nc,path.nonmem,file.mod,fnExtension(file.mod,"lst"),pnm,nc)
                     ## executing from model execution dir.
+                    jobname <- basename(file.mod)
+                    ## qsub does not allow a jobname to start in a numeric
+                    if(grepl("^ *[0-9]",jobname)) {
+                        jobname <- paste0("NMsim_",jobname)
+                    }
                     string.cmd <- sprintf('cd \"%s\"; qsub -pe orte %s -V -N \"%s\" -j y -cwd -b y \"./%s\" -background -parafile=%s [nodes]=%s; cd \"%s\"'
-                                         ,dirname(string.cmd),nc,basename(file.mod)
-                                          ## ,NMsimConf$path.nonmem,basename(file.mod),fnExtension(basename(file.mod),"lst")
+                                         ,dirname(string.cmd)
+                                         ,nc
+                                         ,jobname
                                          ,basename(string.cmd)
-                                         ,basename(pnm),nc,getwd())
+                                          ##,basename(pnm)
+                                         ,getAbsolutePath(pnm)
+                                         ,nc
+                                         ,getwd())
                 }
                 wait <- TRUE
-            } else {
-                if(NMsimConf$system.type=="linux"){
-                    string.cmd <- sprintf("cd \"%s\"; \"./%s\"",dirname(string.cmd),basename(string.cmd))
-                } 
-                if(NMsimConf$system.type=="windows"){
-                    string.cmd <- sprintf("CD \"%s\";call \"%s\"",dirname(string.cmd),basename(string.cmd))
-                }
             }
+            ## } else { 
+            if(NMsimConf$system.type=="linux"){
+                string.cmd <- sprintf("cd \"%s\"; \"./%s\"",dirname(string.cmd),basename(string.cmd))
+            }
+            if(NMsimConf$system.type=="windows"){
+                string.cmd <- sprintf("CD \"%s\";call \"%s\"",dirname(string.cmd),basename(string.cmd))
+            }
+            ## }
         }
         
         if(NMsimConf$system.type=="windows"){
@@ -333,8 +356,12 @@ NMexec <- function(files,file.pattern,dir,sge=TRUE,input.archive,
             
             if(nmquiet) string.cmd <- paste(string.cmd, ">/dev/null 2>&1")
             if(!wait) string.cmd <- paste(string.cmd,"&")
+            if(exists("dir.tmp") && !is.null(dir.tmp)) {          
+                writeTextFile(string.cmd,file.path(dir.tmp,"NMexec_command.txt"))
+            }
             
-            system(string.cmd,ignore.stdout=nmquiet)
+            procres <- system(string.cmd,ignore.stdout=nmquiet)
+            if(procres!=0) stop("Nonmem failed. Exiting.")
         }
     }
 
